function [point_list, Npoints, first_rhumb, first_turn, coverage] = scan(lat1,lon1,lat3,lon3,crs,turn_radius)
%Scanning pattern
% Computes a scanning pattern as a sequence of straight and turn legs
% around a square defined by vertices (lat1, lon1) and (lat2, lon2).
% crs (decimal degrees) is the main direction of the scanning pattern.
% turn_radius (m) is the radius of the turn leg. It can be seen also as the
% semi-area covered in each pass.
% The function returns a 32x2 matrix containing the coordinates of the
% points which define the pattern: the entry point, the destination point
% of each straight segment, the turn center of each turn leg, and the exit
% point in the form [lat(i) lon(i)]. The scanning pattern shall have a
% maximum number of 32 points.
% Npoints is the actual number of points which define the scanning pattern.
% first_rhumb (decimal degrees) is the rhumb of the first straight leg.
% first_turn is the direction of the first turn segment (CW=1, CCW=-1)
% coverage (m) is the turn_radius corrected to optimize the scanning area.
%
% Asensio Lorenzo Sempere 9/2/2016

%Defines
CONSTNPOINTS = 32;
CW = 1;
CCW = -1;
%----------------------------------------------------
% Square obtention
[lat2, lon2, lat4, lon4] = loxo_pt2sq(lat1,lon1,lat3,lon3,crs);

% Obtain the references distances and angles
[dist12, bearing12] = loxo_dist_bearing(lat1,lon1,lat2,lon2);
[dist14, bearing14] = loxo_dist_bearing(lat1,lon1,lat4,lon4);

% Obtain number of LONgitudinal divisions
N_largos_prov = floor(dist14 / (2*turn_radius));

% Correct the number of divisions (even / odd)
if (rem(N_largos_prov,2) == 0) % Even: Ends the wrong corner
    N_largos   = N_largos_prov -1;         % Divisions are reduced
    cob_efectiva = dist14 / (2*N_largos);   % Range is increased
else                      % Odd: Ends the right corner
    N_largos   = N_largos_prov;
    cob_efectiva = dist14 / (2*N_largos);   % Appropiated range is covered
end
Npoints = 2*N_largos;
%----------------------------------------------------
% Definitions

point_list = zeros(CONSTNPOINTS,2);
tempwaypoint = zeros(1,2);
rev_bearing12 = mod(bearing12+180,360);pos = 0;

% Waypoints
[point_list(1,1), point_list(1,2)] = loxo_destination(lat1,lon1,dist14/Npoints,bearing14);
for i = 2:(Npoints-1)
    if rem(i,2)==0  % a?? el punto de tangencia
        if (pos == 0)       % Primero
            [tempwaypoint(1),tempwaypoint(2)] = loxo_destination(lat2,lon2,dist14*(i-1)/Npoints,bearing14);
            [point_list(i,1),point_list(i,2)] = loxo_destination(tempwaypoint(1),tempwaypoint(2),2*cob_efectiva,rev_bearing12);
            pos = 1;
        elseif (pos == 1)   % Tercero
            [tempwaypoint(1),tempwaypoint(2)] = loxo_destination(lat1,lon1,dist14*(i-1)/Npoints,bearing14);
            [point_list(i,1),point_list(i,2)] = loxo_destination(tempwaypoint(1),tempwaypoint(2),2*cob_efectiva,bearing12);
            pos = 0;
        end
    else            % a?? el punto de centro de circunferencia
        if (pos == 1)       % Segundo
            [tempwaypoint(1),tempwaypoint(2)] = loxo_destination(lat2,lon2,dist14*(i-1)/Npoints,bearing14);
            [point_list(i,1),point_list(i,2)] = loxo_destination(tempwaypoint(1),tempwaypoint(2),2*cob_efectiva,rev_bearing12);
        elseif (pos == 0)   % Cuarto
            [tempwaypoint(1),tempwaypoint(2)] = loxo_destination(lat1,lon1,dist14*(i-1)/Npoints,bearing14);
            [point_list(i,1),point_list(i,2)] = loxo_destination(tempwaypoint(1),tempwaypoint(2),2*cob_efectiva,bearing12);
        end
    end
end
[point_list(Npoints,1), point_list(Npoints,2)] = loxo_destination(lat3,lon3,dist14/Npoints,mod(bearing14+180,360));

% Obtain the first turn
objective_brg23 = unwrap(bearing14-bearing12, 180);
if objective_brg23>=0
    first_turn = CW;
else
    first_turn = CCW;
end

% First rhumb
first_rhumb = bearing12;

%Turn radius corrected
coverage = cob_efectiva; %m
end

%% ==================================================
function [lat2,lon2,lat4,lon4] = loxo_pt2sq(lat1,lon1,lat3,lon3,bearing)
% bearing = [0, 360[
% Graphic representation of the resulting points
%  1     main_dist     2
%   o-----------------o
%   |-->->->->->->->\ |
%   |               | |
%   | /<-<-<-<-<-<--/ | secondary_dist
%   | |               |
%   | \->->->->->->-->|
%   o-----------------o
%  4                   3
bearing = mod(bearing,360);

if (bearing == 90 || bearing == 270)
    lat2 = lat1;
    lon2 = lon3;
    lat4 = lat3;
    lon4 = lon1;
elseif (bearing == 0 || bearing == 180)
    lat2 = lat3;
    lon2 = lon1;
    lat4 = lat1;
    lon4 = lon3;
else
    %if (mod(bearing,90) ~= 0)
    [lat2, lon2] = loxo_90intersection(lat1,lon1,lat3,lon3,bearing);
    [lat4, lon4] = loxo_90intersection(lat1,lon1,lat3,lon3,mod(bearing+90,360));
end
end

%% ==================================================
function [lat3,lon3]=loxo_90intersection(lat1,lon1,lat2,lon2,crs13)
lat1 = lat1*pi/180;
lon1 = lon1*pi/180;

lat2 = lat2*pi/180;
lon2 = lon2*pi/180;

m = tand(crs13);
lon3 = ((lon2*m+lon1/m)-log(tan(pi/4+lat1/2)/tan(pi/4+lat2/2)))/(m+1/m);
lat3 = 2*atan(exp((m^2*log(tan(pi/4+lat1/2))+m*(lon2-lon1)+log(tan(pi/4+lat2/2)))/(1+m^2)))-pi/2;

lat3 = lat3*180/pi;
lon3 = lon3*180/pi;
end

%% ==================================================
function [lat_fin, lon_fin] = loxo_destination(lat_ini, lon_ini, dist, brg)
R = 6371000;
lat_ini = lat_ini*pi/180;
lon_ini = lon_ini*pi/180;
brg = brg*pi/180;

alpha = dist/R;
lat_fin = lat_ini + alpha*cos(brg);
dLat = lat_fin - lat_ini;
dLatp = log(tan(pi/4+lat_fin/2)/tan(pi/4+lat_ini/2));
if (brg == pi/2)
    q = cos(lat_ini);
else
    q = dLat/dLatp;
end
dLon = alpha*sin(brg)/q;
lon_fin = mod(lon_ini + dLon + pi, 2*pi)-pi;
if (lat_fin > pi/2)
    lat_fin = pi - lat_fin;
elseif (lat_fin < -pi/2)
    lat_fin = -pi - lat_fin;
end

lat_fin = lat_fin*180/pi;
lon_fin = lon_fin*180/pi;
end

%% ==================================================
function [dist, brg] = loxo_dist_bearing(lat_ini, lon_ini, lat_fin, lon_fin)
lat_ini=lat_ini*pi/180;
lon_ini=lon_ini*pi/180;
lat_fin=lat_fin*pi/180;
lon_fin=lon_fin*pi/180;
Dphi = log(tan(pi/4+lat_fin/2)/tan(pi/4+lat_ini/2));
if lat_ini==lat_fin
    q=cos(lat_ini);
else
    q=(lat_fin-lat_ini)/(Dphi);
end
dlon = (lon_fin-lon_ini);
if dlon > pi
    dlon = dlon-2*pi;
elseif dlon < -pi
    dlon = dlon+2*pi;
end
dist = sqrt((lat_fin-lat_ini)^2 + q^2*dlon^2)*6371000;
brg  = atan2(dlon, (Dphi))*180/pi;
if(brg<0)
    brg = brg + 360;
end
end

%% ==================================================
function out = unwrap(in, limit)
out = in;
if out > limit
    out = out-2*limit;
end
if out < -limit
    out = out+2*limit;
end
end